var _default_vert = require("../Shaders/ccShader_Default_Vert.js");
var _gaussian_blur_frag = require("../Shaders/ccShader_Color");

var AvgBlackFlash = cc.Class({
    extends: cc.Component,

    properties: {
        angle: {
            default: 0,
            tooltip: "角度值 0 ~ 360"
        },
        offset: {
            default: 0,
            tooltip: "偏移值 -1 ~ 1"
        },
        beginColor: cc.color("#ffffff"),
        endColor: cc.color("#ff0000"),
    },

    onLoad: function () {
        this.parameters={
            // time: 0,
            textureSize: {
                // x: GameConfig.winSize.width,
                // y: GameConfig.winSize.height,
                x: this.node.width,
                y: this.node.height,
            },
            beginColor: {
                r: this.beginColor.r / 255,
                g: this.beginColor.g / 255,
                b: this.beginColor.b / 255,
                a: this.beginColor.a / 255,
            },
            endColor: {
                r: this.endColor.r / 255,
                g: this.endColor.g / 255,
                b: this.endColor.b / 255,
                a: this.endColor.a / 255,
            },
        };
        if (cc.sys.isNative) {
            this.initShaderProgram = this.node._sgNode.getGLProgramState();
        }else{
            this.initShaderProgram = this.node._sgNode.getShaderProgram();
        }
        // this.getComponent(cc.Sprite).spriteFrame.getTexture().setTexParameters(
        //     cc.Texture2D.Filter.LINEAR,
        //     cc.Texture2D.Filter.LINEAR,
        //     cc.Texture2D.Filter.GL_CLAMP_TO_EDGE,
        //     cc.Texture2D.Filter.GL_CLAMP_TO_EDGE )
        this._use();
    },

    _use: function()
    {
        this._program = new cc.GLProgram();
        if (cc.sys.isNative) {
            cc.log("use native GLProgram")
            this._program.initWithString(_default_vert, _gaussian_blur_frag);
            this._program.link();
            this._program.updateUniforms();
            this.updateGLParameters();
        }else{
            this._program.initWithVertexShaderByteArray(_default_vert, _gaussian_blur_frag);
            this._program.addAttribute(cc.macro.ATTRIBUTE_NAME_POSITION, cc.macro.VERTEX_ATTRIB_POSITION);
            this._program.addAttribute(cc.macro.ATTRIBUTE_NAME_COLOR, cc.macro.VERTEX_ATTRIB_COLOR);
            this._program.addAttribute(cc.macro.ATTRIBUTE_NAME_TEX_COORD, cc.macro.VERTEX_ATTRIB_TEX_COORDS);
            this._program.link();
            this._program.updateUniforms();
            this.updateGLParameters();
        }

        if (cc.sys.isNative) {
            var glProgram_state = cc.GLProgramState.getOrCreateWithGLProgram(this._program);
            // glProgram_state.setUniformVec2( "resolution", this.parameters.resolution );
            // // glProgram_state.setUniformFloat( "time" , this.parameters.time );
            // glProgram_state.setUniformVec2( "mouse_touch" , this.parameters.mouse );
            glProgram_state.setUniformVec2("u_textureSize" , this.parameters.textureSize);
            glProgram_state.setUniformFloat( "u_angle" , this.angle);
            glProgram_state.setUniformFloat( "u_offset" , this.offset);
            glProgram_state.setUniformVec4( "u_beginColor" , this.parameters.beginColor);
            glProgram_state.setUniformVec4( "u_endColor" , this.parameters.endColor);
        }else{


            // this._resolution = this._program.getUniformLocationForName( "resolution" );
            // // this._time = this._program.getUniformLocationForName( "time" );
            // this._mouse = this._program.getUniformLocationForName( "mouse_touch" );
            this._textureSize = this._program.getUniformLocationForName( "u_textureSize" );
            this._angle = this._program.getUniformLocationForName( "u_angle" );
            this._offset = this._program.getUniformLocationForName( "u_offset" );
            this._endColor = this._program.getUniformLocationForName( "u_endColor" );
            this._beginColor = this._program.getUniformLocationForName( "u_beginColor" );

            // this._program.setUniformLocationWith2f( this._resolution, this.parameters.resolution.x,this.parameters.resolution.y );
            // // this._program.setUniformLocationWith1f( this._time, this.parameters.time );
            // this._program.setUniformLocationWith2f( this._mouse, this.parameters.mouse.x,this.parameters.mouse.y );
            this._program.setUniformLocationWith2f( this._textureSize, this.parameters.textureSize.x, this.parameters.textureSize.y);
            this._program.setUniformLocationWith1f( this._angle, this.angle);
            this._program.setUniformLocationWith1f( this._offset, this.offset);
            this._program.setUniformLocationWith4f( this._endColor, this.parameters.endColor.r, this.parameters.endColor.g, this.parameters.endColor.b, this.parameters.endColor.a);
            this._program.setUniformLocationWith4f( this._beginColor, this.parameters.beginColor.r, this.parameters.beginColor.g, this.parameters.beginColor.b, this.parameters.beginColor.a);
        }

        this.setProgram( this.node._sgNode, this._program );
        
    },
    setProgram:function (node, program) {

        if (cc.sys.isNative) {
            var glProgram_state = cc.GLProgramState.getOrCreateWithGLProgram(program);
            node.setGLProgramState(glProgram_state);
        }else{
            node.setShaderProgram(program);    
        }
        
        var children = node.children;
        if (!children)
            return;
    
        for (var i = 0; i < children.length; i++)
        {
            this.setProgram(children[i], program);
        }
    },
    update(dt) {
        // return;
        if(this._program){

            this._program.use();
            this.updateGLParameters(dt);
            if(cc.sys.isNative){
                var glProgram_state = cc.GLProgramState.getOrCreateWithGLProgram(this._program);
                // glProgram_state.setUniformVec2( "resolution", this.parameters.resolution );
                // // glProgram_state.setUniformFloat( "time", this.parameters.time );    
                // glProgram_state.setUniformVec2( "mouse_touch" , this.parameters.mouse );
                
                glProgram_state.setUniformVec2("u_textureSize" , this.parameters.textureSize);
                glProgram_state.setUniformFloat( "u_angle" , this.angle);
                glProgram_state.setUniformFloat( "u_offset" , this.offset);
                glProgram_state.setUniformVec4( "u_beginColor" , this.parameters.beginColor);
                glProgram_state.setUniformVec4( "u_endColor" , this.parameters.endColor);
            }else{
                // this._program.setUniformLocationWith2f( this._resolution, this.parameters.resolution.x,this.parameters.resolution.y );
                // // this._program.setUniformLocationWith1f( this._time, this.parameters.time );
                // this._program.setUniformLocationWith2f( this._mouse, this.parameters.mouse.x,this.parameters.mouse.x );
                
                this._program.setUniformLocationWith2f( this._textureSize, this.parameters.textureSize.x, this.parameters.textureSize.y);
                this._program.setUniformLocationWith1f( this._angle, this.angle);
                this._program.setUniformLocationWith1f( this._offset, this.offset);
                this._program.setUniformLocationWith4f( this._endColor, this.parameters.endColor.r, this.parameters.endColor.g, this.parameters.endColor.b, this.parameters.endColor.a);
                this._program.setUniformLocationWith4f( this._beginColor, this.parameters.beginColor.r, this.parameters.beginColor.g, this.parameters.beginColor.b, this.parameters.beginColor.a);
            }
        }
    },
    updateGLParameters(dt){
        // this.parameters.time += dt || 0;
    },
    onEnable() {
        if (this._program) {
            
        }
    },
    onDisable() {
        if (this._program) {
            this._program.use();
            // this.parameters.time = 0;
            this.update(0);
        }
    },
});





module.exports = AvgBlackFlash;



